/*
    FreeRTOS V6.1.0 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS books - available as PDF or paperback  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/*
 * This is a very simple demo that demonstrates task and queue usages only.
 * Details of other FreeRTOS features (API functions, tracing features,
 * diagnostic hook functions, memory management, etc.) can be found on the
 * FreeRTOS web site (http://www.FreeRTOS.org) and in the FreeRTOS book.
 * Details of this demo (what it does, how it should behave, etc.) can be found
 * in the accompanying PDF application note.
 *
 *
 * PAC3 added MutEx semaphores for resource (uart) shared access
 *
*/

/* Kernel includes. */
#include "util.h" // Includes FreeRTOS and queue.h
#include "task.h"
#include <string.h>
#include "strutils.h"
#include "wiflyFreeRTOS.h"

#define DEFAULT_STACK_SIZE    400
#define DEFAULT_TASK_PRIORITY 1
#define READ_TASK_PRIORITY    2


/*
 * VariablespvDeflateTask
 *
 */
#define PROPERTY_STRING 1
#define PROPERTY_UCHAR  2
#define PROPERTY_INT    3

#define NUM_PROPERTIES 19

struct _property {
	unsigned char type;
	char *name;
	void *pointer;
};


struct _task {
	char *name;
	void (*prvTask)( void *);
	int stackSize;
	int priority;
};


#define VAR_SMALL_BUFFER   5
#define VAR_MEDIUM_BUFFER 32
#define VAR_BIG_BUFFER    64

/* last command */
static char command[4];
static char property[VAR_MEDIUM_BUFFER];
static char value[VAR_BIG_BUFFER];

static char wsInType[VAR_SMALL_BUFFER];
static char wsOutType[VAR_SMALL_BUFFER];
static char wsInFile[VAR_MEDIUM_BUFFER];
static char wsOutFile[VAR_MEDIUM_BUFFER];
static char wsInBytes[VAR_MEDIUM_BUFFER];
static char wsOutBytes[VAR_MEDIUM_BUFFER];
static char wsInOffset[VAR_MEDIUM_BUFFER];
static char wsOutOffset[VAR_MEDIUM_BUFFER];
static char wsOutTime[VAR_MEDIUM_BUFFER];
static char wsOutRatio[VAR_MEDIUM_BUFFER];
static char wsGetInfo[VAR_MEDIUM_BUFFER];

static char wsInUrl[VAR_BIG_BUFFER];
static char wsOutUrl[VAR_BIG_BUFFER];

static char cpInFileName[VAR_BIG_BUFFER];
static char cpOutFileName[VAR_BIG_BUFFER];

static char cpOperation;
static char cpType[VAR_MEDIUM_BUFFER];
static char cpFormat[VAR_SMALL_BUFFER];
static char cpLevel;

static unsigned long cpBytes = 128L;
static unsigned long cpFileInSize = 0L;
static unsigned long cpFileOutSize = 0L;
static unsigned long cpTime = 0L;

#define READ_BUFFER  128  // 128
#define WRITE_BUFFER 456  // 256+200


struct _property properties[NUM_PROPERTIES] = {
		{PROPERTY_STRING, "wsInType", (void*) &wsInType},
		{PROPERTY_STRING, "wsOutType", (void*) &wsOutType},
		{PROPERTY_STRING, "wsInFile", (void*) &wsInFile},
		{PROPERTY_STRING, "wsOutFile", (void*) &wsOutFile},
		{PROPERTY_STRING, "wsInBytes", (void*) &wsInBytes},
		{PROPERTY_STRING, "wsOutBytes", (void*) &wsOutBytes},
		{PROPERTY_STRING, "wsInOffset", (void*) &wsInOffset},
		{PROPERTY_STRING, "wsOutOffset", (void*) &wsOutOffset},
		{PROPERTY_STRING, "wsOutTime", (void*) &wsOutTime},
		{PROPERTY_STRING, "wsOutRatio", (void*) &wsOutRatio},
		{PROPERTY_STRING, "wsInUrl", (void*) &wsInUrl},
		{PROPERTY_STRING, "wsOutUrl", (void*) &wsOutUrl},
		{PROPERTY_STRING, "wsGetInfo", (void*) &wsGetInfo},
		{PROPERTY_STRING, "cpType", (void*) &cpType},
		{PROPERTY_STRING, "cpFormat", (void*) &cpFormat},
		{PROPERTY_STRING, "cpInFileName", (void*) &cpInFileName},
		{PROPERTY_STRING, "cpOutFileName", (void*) &cpOutFileName},
		{PROPERTY_UCHAR, "cpOperation", (void*) &cpOperation},
		{PROPERTY_UCHAR, "cpLevel", (void*) &cpLevel}
};


static struct _wiflyFreeRTOS wiflyFreeRTOS;

/*
 * Tasks Handles
 */
static xTaskHandle hTskControl;
static xTaskHandle hTskTerminal;
static xTaskHandle hTskRead;
static xTaskHandle hTskWrite;
static xTaskHandle hTskCompress;


/*
 * The tasks.
 */

static void prvControlTask( void *pvParameters );
static void prvTerminalTask( void *pvParameters );
static void pvHTTPReadTask( void *pvParameters );
static void pvHTTPWriteTask( void *pvParameters );
static void pvDeflateTask( void *pvParameters );
static void pvCopyTask( void *pvParameters ); // for debug

/*
 * Private functions
 */
int getPropertyIndex(char *name);
boolean setPropertyByName( char *name, char *value );
char *getPropertyByIndex( int index, char *returnValue );
char *getPropertyByName( char *name, char *returnValue );
void emptyString(char *value);
boolean sendTerminalWriteMessageNoEnd(char *message);
boolean sendTerminalWriteMessage(char *message);
int strcmpi(char *a, char *b);
void connectWLAN();

struct _task *getTaskByName(char *name, struct _task *pTask, int maxTasks);


#define NUM_READ_TASKS 1
struct _task readTask[NUM_READ_TASKS] = {
	{"http", pvHTTPReadTask, 400, READ_TASK_PRIORITY}
};

#define NUM_WRITE_TASKS 1
struct _task writeTask[NUM_WRITE_TASKS]  = {
	{"http", pvHTTPWriteTask, 500, DEFAULT_TASK_PRIORITY}
};

#define NUM_COMPRESS_TASKS 2
struct _task compressTask[NUM_COMPRESS_TASKS]  = {
	{"copy", pvCopyTask, 100, DEFAULT_TASK_PRIORITY},
	{"deflate", pvDeflateTask, 200, DEFAULT_TASK_PRIORITY}
};

/*
 * now()
 * Returns the actual millisecond
 */
unsigned long now()
{
	return (unsigned long) xTaskGetTickCount()/portTICK_RATE_MS;
}



/*-----------------------------------------------------------*/

int main(void)
{
	/* Initialize properties */
	strcpy(wsInType,"http");
	strcpy(wsOutType, wsInType); // Out is the same as In
	strcpy(wsInFile, "wsInFile");
	strcpy(wsOutFile, "wsOutFile");
	strcpy(wsInBytes, "wsInBytes");
	strcpy(wsOutBytes, "wsOutBytes");
	strcpy(wsInOffset, "wsInOffset");
	strcpy(wsOutOffset, "wsOutOffset");
	strcpy(wsOutTime, "wsOutTime");
	strcpy(wsOutRatio, "wsOutRatio");
	strcpy(wsGetInfo, "wsGetInfo");

	strcpy(wsInUrl, "http://www.jrufas.hol.es/readFileBlock.php");
	strcpy(wsOutUrl, "http://www.jrufas.hol.es/writeFileBlock.php");

	cpOperation = 1; // 0 Copy, 1 Compress, 2 Decompress
	strcpy(cpInFileName, "test1.txt");
	strcpy(cpOutFileName, "test1.txt.gz");
	strcpy(cpType, "deflate");
	strcpy(cpFormat,"gzip");
	cpLevel = 9; // 0 - 9 (worst-best) (fast-slow)




	/* Start the Control task */
	xTaskCreate( prvControlTask, ( signed char * ) "Control", DEFAULT_STACK_SIZE, NULL, DEFAULT_TASK_PRIORITY , &hTskControl );

	/* Start the task running. */
	vTaskStartScheduler();

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
}
/*
 * You must put your WLAN SSID and Passphrase
 */
void connectWLAN()
{
	if (!WiFlyFreeRTOSisAssociated(&wiflyFreeRTOS)) {
		WiFlyFreeRTOSjoinAutoWPA(&wiflyFreeRTOS, "WLAN_PFC", "PFC_jvrufas_UOC_2012");
	}
}

//void (*getTaskByName( char *name, struct _task *pTask, int maxTasks )) ( void *pvParameters )

struct _task *getTaskByName(char *name, struct _task *pTask, int maxTasks)
{
	int i;
	for( i=0; i < maxTasks; i++) {
		if(!strcmpi(name, pTask[i].name ))
			return &pTask[i];
	}
	return NULL;
}

void (*getTaskFunctionByName( char *name, struct _task *pTask, int maxTasks ))(void *)
{
	int i;
	for( i=0; i < maxTasks; i++) {
		if(!strcmpi(name, pTask[i].name ))
			return pTask[i].prvTask;
	}
	return NULL;
}

/*-----------------------------------------------------------*/


/*
 * Control Task
 *
 * For memory reasons it can't allocate and free FreeRTOS resources (tasks, queues, etc...)
 *
 */
static void prvControlTask( void *pvParameters )
{
	struct _task *pTask;
	boolean firstExecution = true;

	// Init WiFly and UART for debug
	WiFlyFreeRTOSsetWiFlyUart( &wiflyFreeRTOS, 3);
	WiFlyFreeRTOSsetUartDebug(&wiflyFreeRTOS, 0, 9600, DEBUG_MODE_NONE);

	WiFlyFreeRTOSbegin(&wiflyFreeRTOS);
	connectWLAN();

	/* Create Terminal Task (read/write terminal)*/
	xTaskCreate( prvTerminalTask, ( signed char * ) "Terminal", DEFAULT_STACK_SIZE, NULL, READ_TASK_PRIORITY, &hTskTerminal );

	while(true) {
		if(WiFlyFreeRTOSisAssociated(&wiflyFreeRTOS)) {
			if( WiFlyFreeRTOSterminalListen(&wiflyFreeRTOS) ) {
				cpTime = now();
				if(firstExecution) {
					pTask = getTaskByName( getPropertyByName( "wsInType", NULL ), readTask, NUM_READ_TASKS );
					if(pTask != NULL) {
						xTaskCreate( pTask->prvTask, ( signed char * ) "Read", pTask->stackSize, NULL, pTask->priority , &hTskRead );
						pTask = getTaskByName( getPropertyByName( "cpType", NULL ), compressTask, NUM_COMPRESS_TASKS );
						if(pTask != NULL) {
							xTaskCreate( pTask->prvTask, ( signed char * ) "Compress", pTask->stackSize, NULL, pTask->priority , &hTskCompress );
							pTask = getTaskByName( getPropertyByName( "wsOutType", NULL ), writeTask, NUM_WRITE_TASKS );
							if(pTask != NULL) {
								xTaskCreate( pTask->prvTask, ( signed char * ) "Write", pTask->stackSize, NULL, pTask->priority , &hTskWrite );
								firstExecution = false;
							}
						}
					}
				} else {
					WiFlyFreeRTOSemptyQueues(&wiflyFreeRTOS);
					vTaskResume(hTskRead);
					vTaskResume(hTskCompress);
					vTaskResume(hTskWrite);
				}
				vTaskSuspend(hTskControl); // Suspend until process finished
			}
		} else {
			connectWLAN();
		}
	}
}


/*
 * Get the property Index of a given name
 *
 * returns the index or -1 on error
 */
int getPropertyIndex(char *name) {
	int i;
	for( i=0; i< NUM_PROPERTIES; i++ ) {
		if( !strcmpi(name, properties[i].name) ) {
			return i;//	WiFlyFreeRTOSjoinAutoWPA(&wiflyFreeRTOS, "WLAN_PFC", "PFC_jvrufas_UOC_2012");

		}
	}
	return -1;
}

/*
 * Set property value by its given name
 *
 * returns true on success, or false on error
 */
boolean setPropertyByName( char *name, char *value )
{
	boolean returnValue = false;
	int i = getPropertyIndex(name);
	int iValue;

	if( i > 0){
		returnValue= true;
		if( properties[i].type == PROPERTY_STRING ) {
			strcpy((char *) properties[i].pointer, value);
		} else if(properties[i].type == PROPERTY_UCHAR) {
			iValue = atoi(value);
			if( iValue > 255) iValue = 255;
			if( iValue < 0) iValue = 0;
			*((unsigned char *) properties[i].pointer) = (unsigned char) iValue;
		} else if(properties[i].type == PROPERTY_INT) {boolean//	WiFlyFreeRTOSjoinAutoWPA(&wiflyFreeRTOS, "WLAN_PFC", "PFC_jvrufas_UOC_2012");

			iValue = atoi(value);
			*((int *) properties[i].pointer) = iValue;
		} else {
			returnValue = false;
		}
	}
	return returnValue;
}


/*
 * Gets property value by its given name
 * Parameters:
 *   i:         indexgetPropertyIndex of property
 *   returnValue:  buffer to hold an itoa() string
 *
 * returns an string on success, or NULL on error
 */

char *getPropertyByIndex( int index, char *returnValue )
{
	char *retVal = NULL;
	if( index >= 0){
		if( properties[index].type == PROPERTY_STRING ) {
			retVal = (char *) properties[index].pointer;
		} else if(properties[index].type == PROPERTY_UCHAR) {
			if( returnValue != NULL ) {
				itoa( (int) (*((unsigned char *) properties[index].pointer)), returnValue, 10);
				retVal = returnValue;
			}
		} else if(properties[index].type == PROPERTY_INT) {
			if( returnValue != NULL ) {
				itoa(*((int *) properties[index].pointer), returnValue, 10);
				retVal = returnValue;
			}
		}
	}
	return retVal;
}

/*
 * Gets property value by its given name
 * Parameters:
 *   name:         name of property
 *   returnValue:  buffer to hold an itoa() string
 *
 * returns an string on success, or NULL on error
 */

char *getPropertyByName( char *name, char *returnValue )
{
	return(getPropertyByIndex(getPropertyIndex(name), returnValue));
}


void emptyString(char *value)
{
	value[0] = '\0';
}

boolean sendTerminalWriteMessageNoEnd(char *message)
{
	if( message ) {
		while( *message != '\0' ) {
			if( xQueueSend( wiflyFreeRTOS.xQueueTerminalWrite, message, portMAX_DELAY ) == pdTRUE ) {
				// for debug purposes
			}
			message++;
		}
	}
}

boolean sendTerminalWriteMessage(char *message)
{
	if( message ) {
		while( true ) {
			if( xQueueSend( wiflyFreeRTOS.xQueueTerminalWrite, message, portMAX_DELAY ) == pdTRUE ) {
				// for debug purposes
			}
			if ( *message == '\0')
				break;

			message++;
		}
	}
}

/*
 * Terminal Task
 * Read-Write terminal commands.
 *
 */
static void prvTerminalTask( void *pvParameters )
{
	char input[80];
	char *current = input;
	char *data;
	int set;


	emptyString(current);
	emptyString(command);
	emptyString(property);
	emptyString(value);
	data = NULL;
	set = 0;

	while(true) {
		/* Read terminal read queue */
		if( xQueueReceive( wiflyFreeRTOS.xQueueTerminalRead, current, portMAX_DELAY ) == pdTRUE ) {
			// First Character (no EOS, no Space, no assigned)
			if(*current != '\0' && *current != ' ' && data == NULL) {
				data = current;
				current ++;
				// Space with data assigned
			}else if(*current == ' ' && data != NULL) {
				*current = '\0';
				switch( set ) {
				case 0:
					strncpy(command, data, 4); // First word: command
					break;
				case 1:
					strncpy(property, data, VAR_MEDIUM_BUFFER); //Second word: property name
					break;
				case 2:
					strncpy(value, data, VAR_BIG_BUFFER); // Third word: value
					break;
				};

				set++;
				data = NULL;
				current ++;
				// End of String
			}else if( *current == '\0' ) {
				if(command[0] == '\0' && data != NULL) {
					strncpy(command, data, 4);
				} else if(property[0] == '\0' && data != NULL){
					strncpy(property, data, VAR_MEDIUM_BUFFER);
				} else if(value[0] == '\0' && data != NULL) {
					strncpy(value, data, VAR_BIG_BUFFER);
				}
				if( command[0] != '\0' ) {
					if( !strncmp("set", command, strlen(command))) {
						if( setPropertyByName(property, value) ) {
							sprintf(input, "%s set to %s\n", property, value );
							sendTerminalWriteMessage(input);
						} else {
							sendTerminalWriteMessage("SET command FAILED");
						}
					} else if(!strncmp("get", command, strlen(command))) {
						emptyString(value);
						if(property[0] != '\0') { // there is a property
							data = getPropertyByName(property, value);
							if( data != NULL) {
								sprintf(input, "%s = %s\n", property, data );
								sendTerminalWriteMessage(input);
							} else {
								sendTerminalWriteMessage("GET command FAILED");
							}
						} else { // there is no property: list all
							for( set=0; set < NUM_PROPERTIES; set++) {
								sprintf(input, "%s = %s\n", properties[set].name, getPropertyByIndex(set, value) );
								sendTerminalWriteMessageNoEnd(input);
							}
							emptyString(input);
							sendTerminalWriteMessage(input); // End of message
						}
					} else if(!strncmp("run", command, strlen(command))) {
						sendTerminalWriteMessage("GO");
					} else {
						sendTerminalWriteMessage("ERROR: command not found");
					}
				}
				emptyString(command);
				emptyString(property);
				emptyString(value);
				data = NULL;
				set = 0;
				current = input;
				emptyString(current);
			} else {
				current ++;
			}// if  else if ...
		} //if xQueueRead
	} // while
}


/*
 * Task for reading a file from HTTP Request
 *
 */
static void pvHTTPReadTask( void *pvParameters )
{
	char url[COMMAND_BUFFER_SIZE*2];
	int bytes = cpBytes;
	byte buffer[READ_BUFFER];
	byte *pBuffer;
	unsigned long bytesRemain = 0L;
	unsigned long bytesReaded = 0L;
	unsigned long offset = 0L;
	unsigned long i;
	int data;

	while(true) {
		/*
		 * Get file Size
		 */
		bytesRemain = 0L;
		bytesReaded = 0L;
		offset = 0L;
		bytes = cpBytes;

		sprintf(url, "%s?%s=%s&%s=1", wsInUrl, wsInFile, cpInFileName, wsGetInfo);
		bytesReaded = WiFlyFreeRTOSgetHTTPRequest(&wiflyFreeRTOS, url, buffer, bytes);
		if( bytesReaded > 0 ){
			buffer[bytesReaded] = '\0';
			cpFileInSize=atol(buffer);
		}
		bytesRemain = cpFileInSize;
		/*
		 * Read file blocks
		 */
		while( bytesRemain > 0) {
			if( bytesRemain < bytes ) bytes=bytesRemain;
			pBuffer = buffer;

			sprintf(url, "%s?%s=%s&%s=%d&%s=%d", wsInUrl, wsInFile, cpInFileName, wsInOffset, offset, wsInBytes, bytes);
			bytesReaded = WiFlyFreeRTOSgetHTTPRequest(&wiflyFreeRTOS, url, buffer, bytes);
			for(i=0L; i< bytesReaded; i++) {
				data = (int) *pBuffer;
				pBuffer++;
				offset++;
				if( xQueueSend( wiflyFreeRTOS.xQueueDataRead, &data, portMAX_DELAY ) != pdTRUE ) {
					break; // Cancel on error
				}
			}

			bytesRemain -= bytesReaded;
		}
		data = -1;  // Send EOF
		xQueueSend( wiflyFreeRTOS.xQueueDataRead, &data, portMAX_DELAY );

		// Suspend Task
		vTaskSuspend(NULL);
	}
}


/*
 * Task for writting a file from HTTP Request
 *
 */
static void pvHTTPWriteTask( void *pvParameters )
{
	byte buffer[WRITE_BUFFER];

	byte resultBuffer[COMMAND_BUFFER_SIZE];

	int count = 0;
	int current;
	char hexByte[3];
	boolean eof=false;
	unsigned long offset = 0L;

	while(true) {
		cpFileOutSize=0L;
		eof=false;
		offset = 0L;
		while(true) {
			if( count == 0) {
				sprintf(buffer, "%s?%s=%s&%s=%d&%s=", wsOutUrl, wsOutFile, cpOutFileName, wsOutOffset, offset, wsOutBytes);
			}
			if( xQueueReceive( wiflyFreeRTOS.xQueueDataWrite, &current, portMAX_DELAY ) == pdTRUE ) {
				if( (current>=0) && (current<256) ) { // if not EOF
					sprintf(hexByte, "%02x", current);
					strcat(buffer, hexByte);
					count++;
				} else {
					eof=true;
				}
				if( (count==cpBytes) || eof ) { // if buffer full, send data.
					if( count > 0) {
						current = strlen(buffer);
						WiFlyFreeRTOSgetHTTPRequest(&wiflyFreeRTOS, buffer, resultBuffer, COMMAND_BUFFER_SIZE);
						cpFileOutSize+=count;
					}
					count = 0;
					offset+=cpBytes;
					buffer[0]='\0';
					sprintf(buffer, "%s?%s=%s&%s=%d&%s=", wsOutUrl, wsOutFile, cpOutFileName, wsOutOffset, offset, wsOutBytes);
				}
				if(eof) {
					break;
				}
			}
		}
		// Send Results
		cpTime = now() - cpTime; // milliseconds of process
		sprintf(buffer, "%s?%s=%s.txt&%s=%d&%s=%d", wsOutUrl, wsOutFile, cpOutFileName, wsOutTime, cpTime, wsOutRatio, cpFileOutSize*100/cpFileInSize);
		WiFlyFreeRTOSgetHTTPRequest(&wiflyFreeRTOS, buffer, resultBuffer, COMMAND_BUFFER_SIZE);

		vTaskResume(hTskControl);
		vTaskSuspend(NULL);
	}

}

/*
 * Task for compress a file from xQueueDataRead to xQueueDataWrite
 *
 * Uses gzipLiteFreeRTOS (gziplite adapted)
 */

static void pvDeflateTask( void *pvParameters )
{
	int current;

	while(true) {
		setLevel(cpLevel);
		zip_init(wiflyFreeRTOS.xQueueDataRead, wiflyFreeRTOS.xQueueDataWrite, NULL, NULL, (int) cpFileInSize, -1);
		zip();
		zip_stop();
		vTaskSuspend(NULL);
	}
}

/*
 * Void compress task
 *
 */
static void pvCopyTask( void *pvParameters )
{
	int current;
	while(true) {
		if( xQueueReceive( wiflyFreeRTOS.xQueueDataRead, &current, portMAX_DELAY ) == pdTRUE ) {
			if( xQueueSend( wiflyFreeRTOS.xQueueDataWrite, &current, portMAX_DELAY ) == pdTRUE ) {
				// for debug purposes
			}
			if( (current<0) || (current>255)) {
				vTaskSuspend(NULL);
			}
		}
	}
}





void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */

	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
