/* gzip -- demo program for gzip/gunzip

   Copyright (C) 2010 Gilles Buloz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include "gzip.h"

//#define FROM_MEMORY /* read data from a buffer in memory rather than from a file descriptor */
//#define FROM_ON_THE_FLY /* data to be ziped/unziped is supplied "on the fly" by calling zip/unzip_add_data() */
//#define TO_MEMORY /* write data to a buffer in memory rather than to a file descriptor */

#if defined(FROM_MEMORY) && defined(FROM_ON_THE_FLY)
#error: Options FROM_MEMORY and FROM_ON_THE_FLY can not be set together
#endif

#define DEMO_MEMORY_BUFFER_SIZE (1UL << 20) /* used for input or output to memory */

int main (argc, argv)
    int argc;
    char **argv;
{
#ifdef FROM_ON_THE_FLY
    char in_data[1]; /* size must be at least 1 */
    int count;
#endif
#ifdef FROM_MEMORY
    char in_memory[DEMO_MEMORY_BUFFER_SIZE]; /* at least the size of the input file */
#endif
#ifdef TO_MEMORY
    char out_memory[DEMO_MEMORY_BUFFER_SIZE]; /* at least the size of the output file */
#endif
    int infd = -1;
    int outfd = -1;
    int infilefd = -1;
    int outfilefd = -1;
    char *inptr = NULL;
    char *outptr = NULL;
    int indatasize = -1; /* unlimited */
    int outmaxsize = -1; /* unlimites */
    int rc = 0;

    if (argc <= 2) {
      fprintf(stderr, "Usage : %s <input file> <output file>", argv[0]);
      return -128;
    }

    infilefd = open(argv[1], O_RDONLY);
    if (infilefd < 0) {
      fprintf(stderr, "ERROR : opening input file : %s\n", argv[1]);
      return -129;
    }
    outfilefd = open(argv[2], O_RDWR | O_CREAT | O_TRUNC, 0664);
    if (outfilefd < 0) {
      fprintf(stderr, "ERROR : opening output file : %s\n", argv[2]);
      return -129;
    }

    infd = infilefd;
    outfd = outfilefd;

#ifdef GZIP
    fprintf(stderr, "gziplite started\n");
#else
    fprintf(stderr, "gunziplite started\n");
#endif

#ifdef FROM_ON_THE_FLY
    infd = FD_ON_THE_FLY;
#endif
#ifdef FROM_MEMORY
    /* read input data from stdin and store it in memory */
    infd = FD_MEMORY;
    inptr = in_memory;
#ifdef GZIP
    /* setting indatasize is mandatory when ziping from memory (because size to zip is unknown) */
    indatasize = read(infilefd, in_memory, sizeof(in_memory));
    /* check size of data read to see if the input file is too big for the buffer */
    /* NOTE : to make it simple, report an error as soon as DEMO_MEMORY_BUFFER_SIZE have been read */
    if (indatasize == DEMO_MEMORY_BUFFER_SIZE) {
	fprintf(stderr, "ERROR : Input file is too big to fit in the input buffer of this demo\n");
	fprintf(stderr, "        Please increase DEMO_MEMORY_BUFFER_SIZE in gzip.c\n");
	return -130;
    }
#else
    /* setting indatasize is not mandatory when unziping but can used to check if too much */
    /* input data is read from memory (use indatasize=-1 to disable this check) */
    read(infilefd, in_memory, sizeof(in_memory));
#endif
#endif
#ifdef TO_MEMORY
    outfd = FD_MEMORY;
    outptr = out_memory;
    outmaxsize = DEMO_MEMORY_BUFFER_SIZE;
#endif

#ifdef GZIP
    rc = zip_init(infd, outfd, inptr, outptr, indatasize, outmaxsize);
    if (rc) return rc;
#else
    rc = unzip_init(infd, outfd, inptr, outptr, indatasize, outmaxsize);
    if (rc) return rc;
#endif

#ifdef FROM_ON_THE_FLY /*------------------------------------------------*/
    /* read small blocks of input data and process then on the fly */
#ifdef GZIP
    while ((count = read(infilefd, in_data, sizeof(in_data))) > 0)
	zip_add_data(in_data, count);
    zip_add_data(in_data, 0); // end of data
#else
    while ((count = read(infilefd, in_data, sizeof(in_data))) > 0)
	unzip_add_data(in_data, count);
    unzip_add_data(in_data, 0); // end of data
#endif
#else /*------------------#ifdef FROM_ON_THE_FLY ----------------------*/
    /* normal processing of inout data (all data already available) */
#ifdef GZIP
    /* ZIP */
#ifdef FROM_MEMORY
    /* Example : check size of ziped data before ziping (not mandatory) */
    zip_unzip_restart(1); /* set test mode */
    rc = zip();
    fprintf(stderr, "gziplite test : rc=%d outsize=%d. Now doing real zip ...\n", rc, bytes_out);
    zip_unzip_restart(0); /* set normal mode */
#endif
    rc = zip();
#else /* #ifdef GZIP */
    /* UNZIP */
#ifdef FROM_MEMORY
    /* Example : check zip image validity and unziped data size before unziping (not mandatory) */
    zip_unzip_restart(1); /* set test mode */
    rc = unzip();
    fprintf(stderr, "gunziplite test : rc=%d outsize=%d. Now doing real unzip ...\n", rc, bytes_out);
    zip_unzip_restart(0); /* set normal mode */
#endif
    rc = unzip();
#endif
#endif /*------------------------------------------------*/

#ifdef TO_MEMORY
    /* write output data in memory to stdout */
    write(outfilefd, out_memory, bytes_out);
#endif

#ifdef GZIP
    zip_stop();
    fprintf(stderr, "gziplite done\n");
#else
    unzip_stop();
    fprintf(stderr, "gunziplite done\n");
#endif

    //fprintf(stderr, "in=%d out=%d\n", bytes_in, bytes_out);

    close(infilefd);
    close(outfilefd);

    return rc;
}
