/* gzip.h -- public header file for gzip/gunzip

   Copyright (C) 2010 Gilles Buloz

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#ifndef _GZIP_H_
#define _GZIP_H_

#include <ctype.h>
#include <sys/types.h> /* for off_t */
#include <stdio.h>  /* for EOF (and fprintf if messages/debug enabled) */
#include <string.h> /* for memset, memcmp, memcpy */
#include <unistd.h> /* for read/write */
#include <stdlib.h> /* for malloc */
#include <fcntl.h>  /* for open */

#define DYN_ALLOC /* use dynamic allocation of memory instead of static arrays (saves some memory) */
//#define MAXSEG_64K /* for systems with 64K segment limit */
#define MESSAGES /* print error/warning messages */

#define ZIP_OUTBUFSIZ 512  /* output buffer size. Must be at least 2 */
/* This is also the blocksize used when writing to output */
//#define ZIP_OUTMINBLKSIZE 4 /* minimum blocksize suported by output device/file. MUST BE A POWER OF TWO */
/* Not used when output is to memory */
/* Used to pad the last write to the minimum suported size */
/* If commented out, the minimum blocksize is 1 */
//#define ZIP_VERY_SMALL_MEM_USAGE /* to have a very small memory usage, but with a little bit less compression ratio */

#define UNZIP_INBUFSIZ 512 /* input buffer size. MUST BE A POWER OF TWO. */
/* Not used when input is from memory */
/* Used as blocksize when input if from a device */
/* Must be at least UNZIP_ONTHEFLY_MIN_LOOKAHEAD for on-the-fly input (see below) */
/* For on the fly processing, the input buffer is used as a circular buffer */
#define UNZIP_OUTBUFSIZ 512 /* output buffer size. MUST BE A POWER OF TWO */
/* Not used when output is to memory */
/* The size of this buffer is also the blocksize used when writing to the output file/device (unless forced below) */
/* To no use any buffer (blocksize = 1), comment out the #define above */
/* To check CRC of the gziped data before doing the real gunzip, set this value to at least the window size (WSIZE) */
/* used when the data was gziped; default is 32768. A lower value can be used if the data has been gziped with */
/* gziplite using the option ZIP_VERY_SMALL_MEM_USAGE */
/* This buffer is mandatory if the output device does not support lseek(). In this can set its size to at least */
/* the window size (WSIZE) used when the data was gziped */
//#define UNZIP_OUTMAXBLKSIZE 64 /* maximum blocksize suported by output device/file. MUST BE A DIVISOR OF UNZIP_OUTBUFSIZ */
/* Not used when output is to memory */
/* If commented out, the maximum blocksize is UNZIP_OUTBUFSIZ */
//#define UNZIP_OUTMINBLKSIZE 4 /* minimum blocksize suported by output device/file. MUST BE A POWER OF TWO */
/* Not used when output is to memory */
/* Used to pad the last write to the minimum suported size */
/* If commented out, the minimum blocksize is 1 */
#define UNZIP_ONTHEFLY_MIN_LOOKAHEAD 256 /* minimum amount of data required in input buffer for on-the-fly-unzip */
/* When unzip_add_data() is called, the input buffer is filled with the supplied data */
/* each time the input buffer is full, it is processed till the minimum lookahead is reached */
/* then the input buffer is filled again until all the data supplied has been added */
/* If this value is too low, we get input underrun errors */
#define UNZIP_CODES_DATA_OVERLAY /* overlay data for fixed and dynamic codes */
/* This saves some memory but fixed data must be inited each time a fixed block is processed, */
/* so unzip may be a little bit slower */

#define OF(args)  args

#define FD_MEMORY     (-1) /* set ifd or ofd to this value for input from / output to memory */
#define FD_ON_THE_FLY (-2) /* set ifd to this value for on the fly compression */

/*
 * NOTE for indatasize and outmaxsize for unzip_init() / zip_init() below
 *
 * indatasize : Mandatory when ziping from memory
 *              For gunzip from memory or gzip/gunzip from a file, it is used to
 *              limit the size of data read. An input_error is set if more
 *              data is required to complete. Not used when input is from on-the-fly
 *              Use value -1 for "no limit"
 *
 * outmaxsize : Maximum size allowed to write to output, or -1 for "no limit"
 *              If more data to write is needed, an output_error is set
 */
	/* in unzip.c */
extern int unzip_init OF((int infd, int outfd, char *indataptr, char *outdataptr, int indatasize, int outmaxsize));
extern int unzip OF((void));
extern int unzip_add_data OF((char *in_data, int count));
extern int unzip_stop OF((void));

	/* in zip.c: */
extern int zip_init OF((int infd, int outfd, char *indataptr, char *outdataptr, int indatasize, int outmaxsize));
extern int zip OF((void));
extern int zip_add_data OF((char *in_data, int count)); /* for on the fly compression */
extern int zip_stop OF((void));

	/* in util.c: */
extern void zip_unzip_restart OF((int test_mode));
extern int bytes_in;   /* number of input bytes processed */
extern int bytes_out;  /* number of output bytes written */

#endif /* _GZIP_H_ */
