/* unzip.c -- decompress files in gzip format.

   Copyright (C) 2011 Gilles Buloz
   Copyright (C) 2002-2010 Mark Adler, all rights reserved
   Copyright (C) 1997-1999, 2009 Free Software Foundation, Inc.
   Copyright (C) 1992-1993 Jean-loup Gailly

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

/*
 * The code in this file is derived from the file funzip.c written
 * and put in the public domain by Mark Adler.
 */

/*
 * The main part of this file has been adapted from the puff.c file from
 * Mark Adler (the original file is supplied with the zlib source code) 
 *
 * Copyright (C) 2002-2010 Mark Adler, all rights reserved
 * version 2.1, 4 Apr 2010
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the author be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * Mark Adler    madler@alumni.caltech.edu
 */

#include "util.h"

#define	OLD_GZIP_MAGIC "\037\236" /* Magic header for gzip 0.5 = freeze 1.x */

/* gzip flag byte */
#define ASCII_FLAG   0x01 /* bit 0 set: file probably ascii text */
#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
#define COMMENT      0x10 /* bit 4 set: file comment present */
#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
#define RESERVED     0xC0 /* bit 6,7:   reserved */

#define MAX_WSIZE 0x8000 /* the maximum window size used when gziping data */

#define get_byte()  (inptr < insize ? uinbuf[(on_the_fly_state) ? inptr++ & (UNZIP_INBUFSIZ - 1) : inptr++] : fill_inbuf())

/* Macros for getting two-byte and four-byte header values */
#define SH(p) ((ush)(uch)((p)[0]) | ((ush)(uch)((p)[1]) << 8))
#define LG(p) ((ulg)(SH(p)) | ((ulg)(SH((p)+2)) << 16))

#define INBUFCOUNT (insize - inptr)

static int on_the_fly_done = 0;
static int can_check_crc = 0;

DECLARE(uch, uinbuf,  UNZIP_INBUFSIZ);

#ifdef UNZIP_OUTBUFSIZ
DECLARE(uch, uoutbuf,  UNZIP_OUTBUFSIZ);
#endif

/* ---- from puff.c----
 *
 * Maximums for allocations and loops.  It is not useful to change these --
 * they are fixed by the deflate format.
 */
#define MAXBITS 15              /* maximum bits in a code */
#define MAXLCODES 286           /* maximum number of literal/length codes */
#define MAXDCODES 30            /* maximum number of distance codes */
#define MAXCODES (MAXLCODES+MAXDCODES)  /* maximum codes lengths to read */
#define FIXLCODES 288           /* number of fixed literal/length codes */
/* ---- end of from puff.c---- ***/
#ifdef UNZIP_CODES_DATA_OVERLAY
typedef union /* to save some memory we overlay the structures are we do not need both at the same time */
#else
typedef struct /* allocate both structures, but as the first is a fixed one, it will ony be filled once (faster) */
#endif
{
    struct { /* 1276 bytes */
	short lencnt[MAXBITS+1], lensym[FIXLCODES];
	short distcnt[MAXBITS+1], distsym[MAXDCODES];
	short lengths[FIXLCODES];
    } fixed;
    struct { /* 1328 bytes */
	short lengths[MAXCODES]; /* descriptor code lengths */
	short lencnt[MAXBITS+1], lensym[MAXLCODES]; /* lencode memory */
	short distcnt[MAXBITS+1], distsym[MAXDCODES]; /* distcode memory */
    } dynamic;
} t_codes_data;
DECLARE(t_codes_data, codes_data, 1);

int unzip_init(int infd, int outfd, char *indataptr, char *outdataptr, int indatasize, int outmaxsize)
{
    int rc = 0;

    ifd = infd;
    ofd = outfd;

    input_data_ptr = indataptr;
    output_data_ptr = outdataptr;
    input_data_size = indatasize;
    output_max_size = outmaxsize;

    if (ifd == FD_MEMORY) {
	/* input is in memory so no need to allocate a buffer for input data */
	uinbuf = (uch *)indataptr;
    } else {
	/* allocate a buffer for loading blocks of input data */
	ALLOC(uch, uinbuf,  UNZIP_INBUFSIZ, rc);
    }
#ifdef UNZIP_OUTBUFSIZ
    if (ofd != FD_MEMORY) {
	ALLOC(uch, uoutbuf,  UNZIP_OUTBUFSIZ, rc);
    }
#endif
    ALLOC(t_codes_data, codes_data, 1, rc);

    zip_unzip_restart(0);

    return rc;
}

int unzip_stop(void)
{
    /* Allocate all global buffers (for DYN_ALLOC option) */
    if (ifd != FD_MEMORY) {
	FREE(uinbuf);
    }
#ifdef UNZIP_OUTBUFSIZ
    if (ofd != FD_MEMORY) {
	FREE(uoutbuf);
    }
#endif
    FREE(codes_data);
    return 0;
}

#ifdef UNZIP_OUTBUFSIZ
#  ifndef UNZIP_OUTMAXBLKSIZE
#    define UNZIP_OUTMAXBLKSIZE UNZIP_OUTBUFSIZ
#  endif
#  ifndef UNZIP_OUTMINBLKSIZE
#    define UNZIP_OUTMINBLKSIZE 1
#  endif
static int write_at_blksize(int fd, uch *buf, int size)
{
    int count, rc;
    for (count = 0; count < size; ) {
	rc = write(fd, buf, ((size - count) > UNZIP_OUTMAXBLKSIZE) ? UNZIP_OUTMAXBLKSIZE : size - count);
	if (rc == 0) break;
	if (rc < 0) return rc;
	count += rc;
	buf += rc;
    }
    return count;
}
#endif

static void write_byte(int fd, uch byte)
{
    updcrc(&byte, 1);

    if (test_mode) {
	/* test mode */
	if (fd == FD_MEMORY) {
	    /* only MAX_WSIZE bytes of memory required for test */
	    output_data_ptr[bytes_out & (MAX_WSIZE - 1)] = byte;
	}
#ifdef UNZIP_OUTBUFSIZ
	else
	{
	    /* a circular output buffer is used, so fill it with data byte */
	    int obufindex = bytes_out & (UNZIP_OUTBUFSIZ - 1);
	    uoutbuf[obufindex] = byte;
	}
#endif
	bytes_out++;
	return;
    }

    /* not test mode */

    if ((output_max_size != (off_t)-1) && (bytes_out >= output_max_size)) {
	/* maximum output size reached */
	output_error = 1;
	return;
    }

    if (fd == FD_MEMORY) {
	/* write is to memory */
	output_data_ptr[bytes_out] = byte; /* write data byte */
    } else {
	int rc = 1;
#ifdef UNZIP_OUTBUFSIZ
	/* a circular output buffer is used, so fill it with data byte, */
	/* and write it each time it is full if not in test mode */
	int obufindex = bytes_out & (UNZIP_OUTBUFSIZ - 1);
	uoutbuf[obufindex] = byte;
	if (obufindex == UNZIP_OUTBUFSIZ - 1)
	    rc = write_at_blksize(fd, uoutbuf, UNZIP_OUTBUFSIZ);
#else
	/* write data byte */
	rc = write(fd, &byte, 1);
#endif
	if (rc <= 0) {
	    output_error = 1; /* I/O error or output full */
	    return;
	}
    }

    bytes_out++;
}

static void write_flush(int fd) {
#ifdef UNZIP_OUTBUFSIZ
    /* end of gunzip : flush data still in buffer, and add some pad if needed to match the min blocksize */
    int obufindex = bytes_out & (UNZIP_OUTBUFSIZ - 1);
    while (obufindex & (UNZIP_OUTMINBLKSIZE - 1))
	uoutbuf[obufindex++] = 0;
    if (!test_mode)
	write_at_blksize(fd, uoutbuf, obufindex);
#endif 
}

uch read_written_byte_at_offset(int fd, int offset)
{
    uch byte;
    off_t cur_offset;

    if (fd == FD_MEMORY) {
	/* output is to memory, so just reread the data written */
	if (!test_mode)
	    byte = output_data_ptr[offset];
	else
	    byte = output_data_ptr[offset & (MAX_WSIZE - 1)]; /* only MAX_WSIZE used for test */
    } else {
	/* output is to a file or device */
#ifdef UNZIP_OUTBUFSIZ
	if (bytes_out - offset <= UNZIP_OUTBUFSIZ)
	{
	    /* a buffer is used and requested data is still in the circular buffer */
	    int obufindex = offset & (UNZIP_OUTBUFSIZ - 1);
	    byte = uoutbuf[obufindex];
	    return byte;
	}
#endif
	/* no buffer is used or data is no more in the buffer */
	if (test_mode) {
	    /* as nothing has been written to device/file, it is impossible to reread data, */
	    /* so just return 0 as dummy data. It will not be possible to check CRC, only size */
	    /* will be checked */
	    can_check_crc = 0;
	    return 0;
	}
	/* seek back in the data written to reread data */
	cur_offset = lseek(fd, 0, SEEK_CUR); /* save current offset */
	lseek(fd, offset, SEEK_SET);
	read(fd, &byte, 1);
	lseek(fd, cur_offset, SEEK_SET); /* restore saved offset for next write */
    }

    return byte;
}

/* Fill the input buffer. This is called only when the buffer is empty */
static int fill_inbuf(void)
{
    int rc;
    int len;
    static int input_limited = 0;

    if (ifd == FD_MEMORY) {
	/* error: should not be there as memory buffer is supposed to contain all data */
	msg_print(stderr, "ERROR: actual size of data is greater than specified size\n");
	input_error = 1;
	return 0; /* return something */
    }
    if (ifd == FD_ON_THE_FLY) {
	/* error: should not be there as unzip_add_data() is supposed to have filled uinbuf */
	msg_print(stderr, "ERROR: on the fly input processing underrun\n");
	input_error = 1;
	return 0; /* return something */
    }

    /* input buffer is currently empty, read as much as possible to fill it */
    insize = 0;
    inptr = 0;
    len = UNZIP_INBUFSIZ - insize;
    if (bytes_in == 0) input_limited = 0;
    if (input_limited) {
	/* the previous read has been limited to not read more than input_data_size */
	/* but as we are here again, this means that unzip() needs more data than allowed */
	input_error = 1; /* error : more than input_data_size required */
	return 0;
    }
    if ((input_data_size != (off_t)-1) && (bytes_in + len > input_data_size)) {
	len = input_data_size - bytes_in;
	input_limited = 1;
    }
    do {
	rc = read(ifd, (char *)uinbuf + insize, len);
	if (rc < 0) {input_error = 1; return 0;} /* input I/O error */
	if (rc == 0) {
	    /* no more data available */
	    if (insize) break; /* at least one byte in input buffer -> OK, we can return a byte of data */
	    /* still nothing in buffer -> nothing to return -> error */
	    input_error = 1;
	    return 0;
	}
	len -=rc;
	insize += rc;
	bytes_in += rc;
    } while (insize < UNZIP_INBUFSIZ);

    return uinbuf[inptr++]; /* return a byte of data */
}


/*------------------------- puff.c --------------------------*/

#define local static            /* for local function definitions */
#define NIL ((unsigned char *)0)        /* for no output option */

/* input and output state */
struct state {
    int bitbuf;                 /* bit buffer */
    int bitcnt;                 /* number of bits in bit buffer */
};

/*
 * Return need bits from the input stream.  This always leaves less than
 * eight bits in the buffer.  bits() works properly for need == 0.
 *
 * Format notes:
 *
 * - Bits are stored in bytes from the least significant bit to the most
 *   significant bit.  Therefore bits are dropped from the bottom of the bit
 *   buffer, using shift right, and new bytes are appended to the top of the
 *   bit buffer, using shift left.
 */
local int bits(struct state *s, int need)
{
    long val;           /* bit accumulator (can use up to 20 bits) */

    /* load at least need bits into val */
    val = s->bitbuf;
    while (s->bitcnt < need) {
        val |= (long)(get_byte()) << s->bitcnt;  /* load eight bits */
 	if (input_error) return 0;
	s->bitcnt += 8;
    }

    /* drop need bits and update buffer, always zero to seven bits left */
    s->bitbuf = (int)(val >> need);
    s->bitcnt -= need;

    /* return need bits, zeroing the bits above that */
    return (int)(val & ((1L << need) - 1));
}

/*
 * Process a stored block.
 *
 * Format notes:
 *
 * - After the two-bit stored block type (00), the stored block length and
 *   stored bytes are byte-aligned for fast copying.  Therefore any leftover
 *   bits in the byte that has the last bit of the type, as many as seven, are
 *   discarded.  The value of the discarded bits are not defined and should not
 *   be checked against any expectation.
 *
 * - The second inverted copy of the stored block length does not have to be
 *   checked, but it's probably a good idea to do so anyway.
 *
 * - A stored block can have zero length.  This is sometimes used to byte-align
 *   subsets of the compressed data for random access or partial recovery.
 */
local int stored(struct state *s)
{
    static unsigned len;       /* length of stored block */

    //fprintf(stderr, "S\n"); fflush(stderr);
    if (on_the_fly_state >= 6) goto oth6;

    /* discard leftover bits from current byte (assumes s->bitcnt < 8) */
    s->bitbuf = 0;
    s->bitcnt = 0;

    /* get length and check against its one's complement */
    len = get_byte();
    len |= get_byte() << 8;
    if (get_byte() != (~len & 0xff) ||
        get_byte() != ((~len >> 8) & 0xff))
        return -2;                              /* didn't match complement! */
    if (input_error) return -30;

    /* copy len bytes from in to out */
    if (on_the_fly_state) on_the_fly_state = 6;
    oth6:
    while (len) {
	if (on_the_fly_state && (INBUFCOUNT < UNZIP_ONTHEFLY_MIN_LOOKAHEAD) && !on_the_fly_eofile) {
	    on_the_fly_done = 1;
	    return 0;
	}
	write_byte(ofd, get_byte());
	if (input_error) return -30;
	len--;
    }
    if (on_the_fly_state) on_the_fly_state = 5;

    /* done with a valid stored block */
    return 0;
}

/*
 * Huffman code decoding tables.  count[1..MAXBITS] is the number of symbols of
 * each length, which for a canonical code are stepped through in order.
 * symbol[] are the symbol values in canonical order, where the number of
 * entries is the sum of the counts in count[].  The decoding process can be
 * seen in the function decode() below.
 */
struct huffman {
    short *count;       /* number of symbols of each length */
    short *symbol;      /* canonically ordered symbols */
};

/*
 * Decode a code from the stream s using huffman table h.  Return the symbol or
 * a negative value if there is an error.  If all of the lengths are zero, i.e.
 * an empty code, or if the code is incomplete and an invalid code is received,
 * then -10 is returned after reading MAXBITS bits.
 *
 * Format notes:
 *
 * - The codes as stored in the compressed data are bit-reversed relative to
 *   a simple integer ordering of codes of the same lengths.  Hence below the
 *   bits are pulled from the compressed data one at a time and used to
 *   build the code value reversed from what is in the stream in order to
 *   permit simple integer comparisons for decoding.  A table-based decoding
 *   scheme (as used in zlib) does not need to do this reversal.
 *
 * - The first code for the shortest length is all zeros.  Subsequent codes of
 *   the same length are simply integer increments of the previous code.  When
 *   moving up a length, a zero bit is appended to the code.  For a complete
 *   code, the last code of the longest length will be all ones.
 *
 * - Incomplete codes are handled by this decoder, since they are permitted
 *   in the deflate format.  See the format notes for fixed() and dynamic().
 */
#ifdef SLOW
local int decode(struct state *s, struct huffman *h)
{
    int len;            /* current number of bits in code */
    int code;           /* len bits being decoded */
    int first;          /* first code of length len */
    int count;          /* number of codes of length len */
    int index;          /* index of first code of length len in symbol table */

    code = first = index = 0;
    for (len = 1; len <= MAXBITS; len++) {
        code |= bits(s, 1);             /* get next bit */
	if (input_error) return -1;
        count = h->count[len];
        if (code - count < first)       /* if length len, return symbol */
            return h->symbol[index + (code - first)];
        index += count;                 /* else update for next length */
        first += count;
        first <<= 1;
        code <<= 1;
    }
    return -10;                         /* ran out of codes */
}

/*
 * A faster version of decode() for real applications of this code.   It's not
 * as readable, but it makes puff() twice as fast.  And it only makes the code
 * a few percent larger.
 */
#else /* !SLOW */
local int decode(struct state *s, struct huffman *h)
{
    int len;            /* current number of bits in code */
    int code;           /* len bits being decoded */
    int first;          /* first code of length len */
    int count;          /* number of codes of length len */
    int index;          /* index of first code of length len in symbol table */
    int bitbuf;         /* bits from stream */
    int left;           /* bits left in next or left to process */
    short *next;        /* next number of codes */

    bitbuf = s->bitbuf;
    left = s->bitcnt;
    code = first = index = 0;
    len = 1;
    next = h->count + 1;
    while (1) {
        while (left--) {
            code |= bitbuf & 1;
            bitbuf >>= 1;
            count = *next++;
            if (code - count < first) { /* if length len, return symbol */
                s->bitbuf = bitbuf;
                s->bitcnt = (s->bitcnt - len) & 7;
                return h->symbol[index + (code - first)];
            }
            index += count;             /* else update for next length */
            first += count;
            first <<= 1;
            code <<= 1;
            len++;
        }
        left = (MAXBITS+1) - len;
        if (left == 0) break;
        bitbuf = get_byte();
	if (input_error) return -1;
        if (left > 8) left = 8;
    }
    return -10;                         /* ran out of codes */
}
#endif /* SLOW */

/*
 * Given the list of code lengths length[0..n-1] representing a canonical
 * Huffman code for n symbols, construct the tables required to decode those
 * codes.  Those tables are the number of codes of each length, and the symbols
 * sorted by length, retaining their original order within each length.  The
 * return value is zero for a complete code set, negative for an over-
 * subscribed code set, and positive for an incomplete code set.  The tables
 * can be used if the return value is zero or positive, but they cannot be used
 * if the return value is negative.  If the return value is zero, it is not
 * possible for decode() using that table to return an error--any stream of
 * enough bits will resolve to a symbol.  If the return value is positive, then
 * it is possible for decode() using that table to return an error for received
 * codes past the end of the incomplete lengths.
 *
 * Not used by decode(), but used for error checking, h->count[0] is the number
 * of the n symbols not in the code.  So n - h->count[0] is the number of
 * codes.  This is useful for checking for incomplete codes that have more than
 * one symbol, which is an error in a dynamic block.
 *
 * Assumption: for all i in 0..n-1, 0 <= length[i] <= MAXBITS
 * This is assured by the construction of the length arrays in dynamic() and
 * fixed() and is not verified by construct().
 *
 * Format notes:
 *
 * - Permitted and expected examples of incomplete codes are one of the fixed
 *   codes and any code with a single symbol which in deflate is coded as one
 *   bit instead of zero bits.  See the format notes for fixed() and dynamic().
 *
 * - Within a given code length, the symbols are kept in ascending order for
 *   the code bits definition.
 */
local int construct(struct huffman *h, short *length, int n)
{
    int symbol;         /* current symbol when stepping through length[] */
    int len;            /* current length when stepping through h->count[] */
    int left;           /* number of possible codes left of current length */
    short offs[MAXBITS+1];      /* offsets in symbol table for each length */

    /* count number of codes of each length */
    for (len = 0; len <= MAXBITS; len++)
        h->count[len] = 0;
    for (symbol = 0; symbol < n; symbol++)
        (h->count[length[symbol]])++;   /* assumes lengths are within bounds */
    if (h->count[0] == n)               /* no codes! */
        return 0;                       /* complete, but decode() will fail */

    /* check for an over-subscribed or incomplete set of lengths */
    left = 1;                           /* one possible code of zero length */
    for (len = 1; len <= MAXBITS; len++) {
        left <<= 1;                     /* one more bit, double codes left */
        left -= h->count[len];          /* deduct count from possible codes */
        if (left < 0) return left;      /* over-subscribed--return negative */
    }                                   /* left > 0 means incomplete */

    /* generate offsets into symbol table for each length for sorting */
    offs[1] = 0;
    for (len = 1; len < MAXBITS; len++)
        offs[len + 1] = offs[len] + h->count[len];

    /*
     * put symbols in table sorted by length, by symbol order within each
     * length
     */
    for (symbol = 0; symbol < n; symbol++)
        if (length[symbol] != 0)
            h->symbol[offs[length[symbol]]++] = symbol;

    /* return zero for complete set, positive for incomplete set */
    return left;
}

/*
 * Decode literal/length and distance codes until an end-of-block code.
 *
 * Format notes:
 *
 * - Compressed data that is after the block type if fixed or after the code
 *   description if dynamic is a combination of literals and length/distance
 *   pairs terminated by and end-of-block code.  Literals are simply Huffman
 *   coded bytes.  A length/distance pair is a coded length followed by a
 *   coded distance to represent a string that occurs earlier in the
 *   uncompressed data that occurs again at the current location.
 *
 * - Literals, lengths, and the end-of-block code are combined into a single
 *   code of up to 286 symbols.  They are 256 literals (0..255), 29 length
 *   symbols (257..285), and the end-of-block symbol (256).
 *
 * - There are 256 possible lengths (3..258), and so 29 symbols are not enough
 *   to represent all of those.  Lengths 3..10 and 258 are in fact represented
 *   by just a length symbol.  Lengths 11..257 are represented as a symbol and
 *   some number of extra bits that are added as an integer to the base length
 *   of the length symbol.  The number of extra bits is determined by the base
 *   length symbol.  These are in the static arrays below, lens[] for the base
 *   lengths and lext[] for the corresponding number of extra bits.
 *
 * - The reason that 258 gets its own symbol is that the longest length is used
 *   often in highly redundant files.  Note that 258 can also be coded as the
 *   base value 227 plus the maximum extra value of 31.  While a good deflate
 *   should never do this, it is not an error, and should be decoded properly.
 *
 * - If a length is decoded, including its extra bits if any, then it is
 *   followed a distance code.  There are up to 30 distance symbols.  Again
 *   there are many more possible distances (1..32768), so extra bits are added
 *   to a base value represented by the symbol.  The distances 1..4 get their
 *   own symbol, but the rest require extra bits.  The base distances and
 *   corresponding number of extra bits are below in the static arrays dist[]
 *   and dext[].
 *
 * - Literal bytes are simply written to the output.  A length/distance pair is
 *   an instruction to copy previously uncompressed bytes to the output.  The
 *   copy is from distance bytes back in the output stream, copying for length
 *   bytes.
 *
 * - Distances pointing before the beginning of the output data are not
 *   permitted.
 *
 * - Overlapped copies, where the length is greater than the distance, are
 *   allowed and common.  For example, a distance of one and a length of 258
 *   simply copies the last byte 258 times.  A distance of four and a length of
 *   twelve copies the last four bytes three times.  A simple forward copy
 *   ignoring whether the length is greater than the distance or not implements
 *   this correctly.  You should not use memcpy() since its behavior is not
 *   defined for overlapped arrays.  You should not use memmove() or bcopy()
 *   since though their behavior -is- defined for overlapping arrays, it is
 *   defined to do the wrong thing in this case.
 */
local int codes(struct state *s,
                struct huffman *lencode,
                struct huffman *distcode)
{
    static int symbol;         /* decoded symbol */
    static int len;            /* length for copy */
    static unsigned dist;      /* distance for copy */
    static const short lens[29] = { /* Size base for length codes 257..285 */
        3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
        35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258};
    static const short lext[29] = { /* Extra bits for length codes 257..285 */
        0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
        3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0};
    static const short dists[30] = { /* Offset base for distance codes 0..29 */
        1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
        257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
        8193, 12289, 16385, 24577};
    static const short dext[30] = { /* Extra bits for distance codes 0..29 */
        0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
        7, 7, 8, 8, 9, 9, 10, 10, 11, 11,
        12, 12, 13, 13};

    /* decode literals and length/distance pairs */
    do {
	if (on_the_fly_state && (INBUFCOUNT < UNZIP_ONTHEFLY_MIN_LOOKAHEAD) && !on_the_fly_eofile) {
	    on_the_fly_done = 1;
	    return 0;
	}

        symbol = decode(s, lencode);
        if (symbol < 0) return symbol;  /* invalid symbol */
        if (symbol < 256) {             /* literal: symbol is the byte */
            /* write out the literal */
	    write_byte(ofd, symbol);
        }
        else if (symbol > 256) {        /* length */
            /* get and compute length */
            symbol -= 257;
            if (symbol >= 29) return -10;       /* invalid fixed code */
            len = lens[symbol] + bits(s, lext[symbol]);

            /* get and check distance */
            symbol = decode(s, distcode);
            if (symbol < 0) return symbol;      /* invalid symbol */
            dist = dists[symbol] + bits(s, dext[symbol]);
	    if (input_error) return -1;
#ifndef INFLATE_ALLOW_INVALID_DISTANCE_TOOFAR_ARRR
            if (dist > bytes_out)
                return -11;     /* distance too far back */
#endif

            /* copy length bytes from distance bytes back */
	    while (len--) {
		write_byte(ofd,
#ifdef INFLATE_ALLOW_INVALID_DISTANCE_TOOFAR_ARRR
			   dist > bytes_out ? 0 :
#endif
			   read_written_byte_at_offset(ofd, bytes_out - dist));
	    }
        }
    } while (symbol != 256);            /* end of block symbol */

    /* done with a valid fixed or dynamic block */
    return 0;
}

/*
 * Process a fixed codes block.
 *
 * Format notes:
 *
 * - This block type can be useful for compressing small amounts of data for
 *   which the size of the code descriptions in a dynamic block exceeds the
 *   benefit of custom codes for that block.  For fixed codes, no bits are
 *   spent on code descriptions.  Instead the code lengths for literal/length
 *   codes and distance codes are fixed.  The specific lengths for each symbol
 *   can be seen in the "for" loops below.
 *
 * - The literal/length code is complete, but has two symbols that are invalid
 *   and should result in an error if received.  This cannot be implemented
 *   simply as an incomplete code since those two symbols are in the "middle"
 *   of the code.  They are eight bits long and the longest literal/length\
 *   code is nine bits.  Therefore the code must be constructed with those
 *   symbols, and the invalid symbols must be detected after decoding.
 *
 * - The fixed distance codes also have two invalid symbols that should result
 *   in an error if received.  Since all of the distance codes are the same
 *   length, this can be implemented as an incomplete code.  Then the invalid
 *   codes are detected while decoding.
 */
local int fixed(struct state *s)
{
    static int virgin = 1;
    static struct huffman lencode, distcode;

    //fprintf(stderr, "F\n"); fflush(stderr);
    if (on_the_fly_state >= 6) goto oth6;

    /* build fixed huffman tables if needed */
    if (virgin) {
        int symbol;
        short *lengths = codes_data->fixed.lengths;

        /* init lencode and distcode structure pointers (must be done before calling construct()) */
        lencode.count = codes_data->fixed.lencnt;
        lencode.symbol = codes_data->fixed.lensym;
        distcode.count = codes_data->fixed.distcnt;
        distcode.symbol = codes_data->fixed.distsym;

        /* literal/length table */
        for (symbol = 0; symbol < 144; symbol++)
            lengths[symbol] = 8;
        for (; symbol < 256; symbol++)
            lengths[symbol] = 9;
        for (; symbol < 280; symbol++)
            lengths[symbol] = 7;
        for (; symbol < FIXLCODES; symbol++)
            lengths[symbol] = 8;
        construct(&lencode, lengths, FIXLCODES);

        /* distance table */
        for (symbol = 0; symbol < MAXDCODES; symbol++)
            lengths[symbol] = 5;
        construct(&distcode, lengths, MAXDCODES);

#ifndef UNZIP_CODES_DATA_OVERLAY
        /* do this just once as codes will never be overwritten */
        virgin = 0;
#endif
    }

    /* decode data until end-of-block code */
    {
	int err;
	if (on_the_fly_state) on_the_fly_state = 6;
        oth6:
	err = codes(s, &lencode, &distcode);
	if (on_the_fly_done) return 0;
	if (on_the_fly_state) on_the_fly_state = 5;
	return err;
    }
}

/*
 * Process a dynamic codes block.
 *
 * Format notes:
 *
 * - A dynamic block starts with a description of the literal/length and
 *   distance codes for that block.  New dynamic blocks allow the compressor to
 *   rapidly adapt to changing data with new codes optimized for that data.
 *
 * - The codes used by the deflate format are "canonical", which means that
 *   the actual bits of the codes are generated in an unambiguous way simply
 *   from the number of bits in each code.  Therefore the code descriptions
 *   are simply a list of code lengths for each symbol.
 *
 * - The code lengths are stored in order for the symbols, so lengths are
 *   provided for each of the literal/length symbols, and for each of the
 *   distance symbols.
 *
 * - If a symbol is not used in the block, this is represented by a zero as
 *   as the code length.  This does not mean a zero-length code, but rather
 *   that no code should be created for this symbol.  There is no way in the
 *   deflate format to represent a zero-length code.
 *
 * - The maximum number of bits in a code is 15, so the possible lengths for
 *   any code are 1..15.
 *
 * - The fact that a length of zero is not permitted for a code has an
 *   interesting consequence.  Normally if only one symbol is used for a given
 *   code, then in fact that code could be represented with zero bits.  However
 *   in deflate, that code has to be at least one bit.  So for example, if
 *   only a single distance base symbol appears in a block, then it will be
 *   represented by a single code of length one, in particular one 0 bit.  This
 *   is an incomplete code, since if a 1 bit is received, it has no meaning,
 *   and should result in an error.  So incomplete distance codes of one symbol
 *   should be permitted, and the receipt of invalid codes should be handled.
 *
 * - It is also possible to have a single literal/length code, but that code
 *   must be the end-of-block code, since every dynamic block has one.  This
 *   is not the most efficient way to create an empty block (an empty fixed
 *   block is fewer bits), but it is allowed by the format.  So incomplete
 *   literal/length codes of one symbol should also be permitted.
 *
 * - If there are only literal codes and no lengths, then there are no distance
 *   codes.  This is represented by one distance code with zero bits.
 *
 * - The list of up to 286 length/literal lengths and up to 30 distance lengths
 *   are themselves compressed using Huffman codes and run-length encoding.  In
 *   the list of code lengths, a 0 symbol means no code, a 1..15 symbol means
 *   that length, and the symbols 16, 17, and 18 are run-length instructions.
 *   Each of 16, 17, and 18 are follwed by extra bits to define the length of
 *   the run.  16 copies the last length 3 to 6 times.  17 represents 3 to 10
 *   zero lengths, and 18 represents 11 to 138 zero lengths.  Unused symbols
 *   are common, hence the special coding for zero lengths.
 *
 * - The symbols for 0..18 are Huffman coded, and so that code must be
 *   described first.  This is simply a sequence of up to 19 three-bit values
 *   representing no code (0) or the code length for that symbol (1..7).
 *
 * - A dynamic block starts with three fixed-size counts from which is computed
 *   the number of literal/length code lengths, the number of distance code
 *   lengths, and the number of code length code lengths (ok, you come up with
 *   a better name!) in the code descriptions.  For the literal/length and
 *   distance codes, lengths after those provided are considered zero, i.e. no
 *   code.  The code length code lengths are received in a permuted order (see
 *   the order[] array below) to make a short code length code length list more
 *   likely.  As it turns out, very short and very long codes are less likely
 *   to be seen in a dynamic code description, hence what may appear initially
 *   to be a peculiar ordering.
 *
 * - Given the number of literal/length code lengths (nlen) and distance code
 *   lengths (ndist), then they are treated as one long list of nlen + ndist
 *   code lengths.  Therefore run-length coding can and often does cross the
 *   boundary between the two sets of lengths.
 *
 * - So to summarize, the code description at the start of a dynamic block is
 *   three counts for the number of code lengths for the literal/length codes,
 *   the distance codes, and the code length codes.  This is followed by the
 *   code length code lengths, three bits each.  This is used to construct the
 *   code length code which is used to read the remainder of the lengths.  Then
 *   the literal/length code lengths and distance lengths are read as a single
 *   set of lengths using the code length codes.  Codes are constructed from
 *   the resulting two sets of lengths, and then finally you can start
 *   decoding actual compressed data in the block.
 *
 * - For reference, a "typical" size for the code description in a dynamic
 *   block is around 80 bytes.
 */
local int dynamic(struct state *s)
{
    static int nlen, ndist, ncode;             /* number of lengths in descriptor */
    static int index;                          /* index of lengths[] */
    static int err;                            /* construct() return value */
    static short *lengths;              /* descriptor code lengths */
    struct huffman lencode, distcode;   /* length and distance codes */
    static const short order[19] =      /* permutation of code length codes */
        {16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15};

    //fprintf(stderr, "D\n"); fflush(stderr);
    if (on_the_fly_state >= 6) goto oth6;

    /* init structure pointers */
    lengths = codes_data->dynamic.lengths;
    lencode.count = codes_data->dynamic.lencnt;
    lencode.symbol = codes_data->dynamic.lensym;
    distcode.count = codes_data->dynamic.distcnt;
    distcode.symbol = codes_data->dynamic.distsym;

    /* get number of lengths in each table, check lengths */
    nlen = bits(s, 5) + 257;
    ndist = bits(s, 5) + 1;
    ncode = bits(s, 4) + 4;
    if (input_error) return -30;
    if (nlen > MAXLCODES || ndist > MAXDCODES)
        return -3;                      /* bad counts */

    /* read code length code lengths (really), missing lengths are zero */
    for (index = 0; index < ncode; index++) {
        lengths[order[index]] = bits(s, 3);
	if (input_error) return -30;
    }
    for (; index < 19; index++)
        lengths[order[index]] = 0;

    /* build huffman table for code lengths codes (use lencode temporarily) */
    err = construct(&lencode, lengths, 19);
    if (err != 0) return -4;            /* require complete code set here */

    /* read length/literal and distance code length tables */
    index = 0;
    while (index < nlen + ndist) {
        int symbol;             /* decoded value */
        int len;                /* last length to repeat */

        symbol = decode(s, &lencode);
	if (input_error) return -30;
        if (symbol < 16)                /* length in 0..15 */
            lengths[index++] = symbol;
        else {                          /* repeat instruction */
            len = 0;                    /* assume repeating zeros */
            if (symbol == 16) {         /* repeat last length 3..6 times */
                if (index == 0) return -5;      /* no last length! */
                len = lengths[index - 1];       /* last length */
                symbol = 3 + bits(s, 2);
            }
            else if (symbol == 17)      /* repeat zero 3..10 times */
                symbol = 3 + bits(s, 3);
            else                        /* == 18, repeat zero 11..138 times */
                symbol = 11 + bits(s, 7);
	    if (input_error) return -30;
            if (index + symbol > nlen + ndist)
                return -6;              /* too many lengths! */
            while (symbol--)            /* repeat last or zero symbol times */
                lengths[index++] = len;
        }
    }

    /* check for end-of-block code -- there better be one! */
    if (lengths[256] == 0)
        return -9;

    /* build huffman table for literal/length codes */
    err = construct(&lencode, lengths, nlen);
    if (err < 0 || (err > 0 && nlen - lencode.count[0] != 1))
        return -7;      /* only allow incomplete codes if just one code */

    /* build huffman table for distance codes */
    err = construct(&distcode, lengths + nlen, ndist);
    if (err < 0 || (err > 0 && ndist - distcode.count[0] != 1))
        return -8;      /* only allow incomplete codes if just one code */

    /* decode data until end-of-block code */
    {
	int err;
	if (on_the_fly_state) on_the_fly_state = 6;
        oth6:
	err = codes(s, &lencode, &distcode);
	if (on_the_fly_done) return 0;
	if (on_the_fly_state) on_the_fly_state = 5;
	return err;
    }
}

local int inflate()
/* decompress an inflated entry */
{
    struct state s;             /* input/output state */
    static int last, type;      /* block information */
    static int err;             /* return value */

    if (on_the_fly_state >= 4) goto oth4;
 
    s.bitbuf = 0;
    s.bitcnt = 0;

    /* process blocks until last block or error */
    do {
	last = bits(&s, 1);         /* one if last block */
	type = bits(&s, 2);         /* block type 0..3 */
	if (input_error) return -30;

	if (on_the_fly_state) on_the_fly_state = 4;
        oth4:

	err = type == 0 ? stored(&s) :
	    (type == 1 ? fixed(&s) :
	     (type == 2 ? dynamic(&s) :
	      -1));               /* type == 3, invalid */

	if (input_error) return -30;
	if (on_the_fly_done) return 0;
	if (err != 0) break;        /* return with error */
    } while (!last);

    if (err == 0)
	write_flush(ofd);

    return err;
}


/*------------------------- unzip.c --------------------------*/

/* ========================================================================
 * Check the magic number of the input file.
 * Return the compression method, or a negative value in case of error
 * Set inptr to the offset of the next byte to be processed.
 * This function may be called repeatedly for an input file consisting
 * of several contiguous gzip'ed members.
 * IN assertions: there is at least one remaining compressed member.
 *   If the member is a zip file, it must be the only one.
 */
static int get_method(void)
{
    uch flags;     /* compression flags */
    char magic[2]; /* magic header */
    ulg stamp;     /* time stamp */

    magic[0] = (char)get_byte();
    magic[1] = (char)get_byte();
    method = -1; /* unknown yet */

    /* assume multiple members in gzip file except for record oriented I/O */
    if (memcmp(magic, GZIP_MAGIC, 2) == 0
        || memcmp(magic, OLD_GZIP_MAGIC, 2) == 0) {

	method = (int)get_byte();
	if (method != DEFLATED) {
	    msg_print(stderr,
		      "ERROR: unknown method %d -- not supported\n", method);
	    return -17;
	}
	flags  = (uch)get_byte();

	if ((flags & ENCRYPTED) != 0) {
	    msg_print(stderr,
		      "ERROR: input is encrypted -- not supported\n");
	    return -18;
	}
	if ((flags & CONTINUATION) != 0) {
	    msg_print(stderr,
		      "ERROR: input is a multi-part gzip file -- not supported\n");
	    return -19;
	}
	if ((flags & RESERVED) != 0) {
	    msg_print(stderr,
		      "ERROR: input has flags 0x%x -- not supported\n", flags);
	    return -20;
	}
	stamp  = (ulg)get_byte();
	stamp |= ((ulg)get_byte()) << 8;
	stamp |= ((ulg)get_byte()) << 16;
	stamp |= ((ulg)get_byte()) << 24;

	(void)get_byte();  /* Ignore extra flags for the moment */
	(void)get_byte();  /* Ignore OS type for the moment */

	if ((flags & CONTINUATION) != 0) {
	    unsigned part = (unsigned)get_byte();
	    part |= ((unsigned)get_byte())<<8;
	    msg_print(stderr,"WARNING: part number %u\n", part);
	}
	if ((flags & EXTRA_FIELD) != 0) {
	    unsigned len = (unsigned)get_byte();
	    len |= ((unsigned)get_byte())<<8;
	    msg_print(stderr,"WARNING: extra field of %u bytes ignored\n", len);
	    while (len--) (void)get_byte();
	}

	/* Skip original file name */
	if ((flags & ORIG_NAME) != 0) {
		char c;
		do {c=get_byte();} while (c != 0);
	}

	/* Discard file comment if any */
	if ((flags & COMMENT) != 0) {
	    while (get_byte() != 0) /* null */ ;
	}
    }

    if (method >= 0) return method;
    msg_print (stderr, "ERROR: not in gzip format\n");

    return -16;
}

/* ===========================================================================
 * Unzip in to out.  This routine works gzip files.
 *
 * IN assertions: the buffer inbuf contains already the beginning of
 *   the compressed data, from offsets inptr to insize-1 included.
 *   The magic header has already been checked. The output buffer is cleared.
 *
 * Return value
 *   0:  successful inflate
 *  -1:  invalid block type (type == 3)
 *  -2:  stored block length did not match one's complement
 *  -3:  dynamic block code description: too many length or distance codes
 *  -4:  dynamic block code description: code lengths codes incomplete
 *  -5:  dynamic block code description: repeat lengths with no first length
 *  -6:  dynamic block code description: repeat more than specified lengths
 *  -7:  dynamic block code description: invalid literal/length code lengths
 *  -8:  dynamic block code description: invalid distance code lengths
 *  -9:  dynamic block code description: missing end-of-block code
 * -10:  invalid literal/length or distance code in fixed or dynamic block
 * -11:  distance is too far back in fixed or dynamic block
 * -16 : not in gzip format
 * -17 : unknown gzip method
 * -18 : input is encrypted : not supported
 * -19 : input is a multi-part gzip file : not supported
 * -20 : input has flags 0x%x : not supported
 * -21 : bad output crc
 * -22 : bad ouput size
 * -30 : input error (I/O error or not enough data)
 */
static int _unzip(void)
{
    static ulg orig_crc;       /* original crc */
    static ulg orig_len;       /* original uncompressed length */
    static uch buf[8];
    static int n;

    if (on_the_fly_state >= 3) goto oth3;

    orig_crc = 0;
    orig_len = 0;

    updcrc(NULL, 0);           /* initialize crc */

    /* Decompress */
    if (method == DEFLATED)  {
	int res;
	if (on_the_fly_state) on_the_fly_state = 3;
        oth3:
	res = inflate();
	if (input_error || output_error) goto io_error;
	if (res) {
	    msg_print(stderr,"ERROR: invalid compressed data--format violated\n");
	    return -11;
	}
	if (on_the_fly_done) return 0;
    } else {
	msg_print(stderr,"ERROR: internal error, invalid method\n");
	return -12;
    }

    /* crc32  (see algorithm.doc)
     * uncompressed input size modulo 2^32
     */
    for (n = 0; n < 8; n++) {
      buf[n] = (uch)get_byte(); /* may cause an error if EOF */
    }
    orig_crc = LG(buf);
    orig_len = LG(buf+4);

    io_error:
    if (input_error) {
	msg_print(stderr, "ERROR: input I/O error or maximum input size reached\n");
	return -15;
    }
    if (output_error) {
	msg_print(stderr, "ERROR: output I/O error or maximum output size reached\n");
	return -16;
    }

    /* Validate decompression */
    if (can_check_crc) {
	if (orig_crc != updcrc(buf, 0)) {
	    msg_print(stderr, "ERROR: invalid compressed data--crc error\n");
	    return -21;
	}
    } else {
	msg_print(stderr, "WARNING: can't check CRC (output buffer size too low); only length is checked.\n");
    }
    if (orig_len != (ulg)(bytes_out & 0xffffffff)) {
	msg_print(stderr, "ERROR: invalid compressed data--length error\n");
	return -22;
    }

    return 0;
}

int unzip(void)
{
    if (on_the_fly_state >= 2) goto oth2;
    can_check_crc = 1;
    method = get_method();
    if (method < 0)
	return method;
    if (on_the_fly_state) on_the_fly_state = 2;
    oth2:
    return _unzip();
}

/* for "on the fly" unzip : add some data to unzip */
int unzip_add_data(char *in_data, int count)
{
    int rc;

    if (input_error || output_error) return -30;
    if (!on_the_fly_state) {
	if (count == 0)
	    return 0; /* end of data at first call -> nothing to do */
	on_the_fly_state = 1;
    }
    if (on_the_fly_eofile) return 0; /* end of data already processed */
    if (count == 0) on_the_fly_eofile = 1; /* end of data */

    on_the_fly_data_size = count;
    on_the_fly_data_ptr = in_data;

    if (!on_the_fly_eofile) {
	/* we have some data to process, add to the input buffer */
	while (on_the_fly_data_size > 0) {
	    /* still have some data to add, add it until buffer full or no more data to add */
	    while (INBUFCOUNT < UNZIP_INBUFSIZ) {
		if (on_the_fly_data_size == 0)
		    return 0; /* all data added */
		uinbuf[insize++ & (UNZIP_INBUFSIZ - 1)] = *on_the_fly_data_ptr;
		on_the_fly_data_ptr++;
		on_the_fly_data_size--;
	    }
	    /* unzip as much as possible */
	    on_the_fly_done = 0;
	    rc = unzip();
	    if (rc) break;
	}
    } else {
	/* no more data to unzip -> process data already in buffer */
	on_the_fly_done = 0;
	rc = unzip();
    }

    return rc;
}
