/***********************************************************************
 * $Id::                                                               $
 *
 * Project:	printf: Simple printf library for LPCXpresso 1700
 * File:	printf.c
 * Description:
 * 			printf functions for use on LPCXpresso 1700
 *
 * 			uartPrintf()
 *
 *  Original code: http://www.menie.org/georges/embedded/printf-stdarg.html
 *  Modified on:   20/10/2012
 *           By:   Javier Rufas
 **********************************************************************/

#include "LPC17xx.h"
#include "type.h"
#include "uart.h"
#include "printf.h"

static int print(struct _uart *uart, const char *format, va_list args );

static void printchar(struct _uart *uart, int c)
{
	UARTsendBytes( uart, (uint8_t *) &c, (uint32_t) 1 );
}

/*
 *  uartPrintf()
 *  For calling like printf().
 */
int uartPrintf(struct _uart *uart, const char *format, ...)
{
        va_list args;
        int ret = 0;

       	va_start( args, format );
       	ret=print( uart, format, args );
       	va_end( args );
        return ret; /* Fail opening UART port */
}


/*
 *  uartPrint()
 *  For calling with a va_list (don't take care of va_start neither va_end).
 */
int uartPrint(struct _uart *uart, const char *format, va_list args)
{
        int ret = 0;

       	ret=print( uart, format, args );
        return ret; /* Fail opening UART port */
}

/*
	Copyright 2001, 2002 Georges Menie (www.menie.org)

	The only change made has been "char **out" for "int port"
*/


#define PAD_RIGHT 1
#define PAD_ZERO 2

static int prints(struct _uart *uart, const char *string, int width, int pad)
{
	register int pc = 0, padchar = ' ';

	if (width > 0) {
		register int len = 0;
		register const char *ptr;
		for (ptr = string; *ptr; ++ptr) ++len;
		if (len >= width) width = 0;
		else width -= len;
		if (pad & PAD_ZERO) padchar = '0';
	}
	if (!(pad & PAD_RIGHT)) {
		for ( ; width > 0; --width) {
			printchar (uart, padchar);
			++pc;
		}
	}
	for ( ; *string ; ++string) {
		printchar (uart, *string);
		++pc;
	}
	for ( ; width > 0; --width) {
		printchar (uart, padchar);
		++pc;
	}

	return pc;
}

/* the following should be enough for 32 bit int */
#define PRINT_BUF_LEN 12

static int printi(struct _uart *uart, int i, int b, int sg, int width, int pad, int letbase)
{
	char print_buf[PRINT_BUF_LEN];
	register char *s;
	register int t, neg = 0, pc = 0;
	register unsigned int u = i;

	if (i == 0) {
		print_buf[0] = '0';
		print_buf[1] = '\0';
		return prints (uart, print_buf, width, pad);
	}

	if (sg && b == 10 && i < 0) {
		neg = 1;
		u = -i;
	}

	s = print_buf + PRINT_BUF_LEN-1;
	*s = '\0';

	while (u) {
		t = u % b;
		if( t >= 10 )
			t += letbase - '0' - 10;
		*--s = t + '0';
		u /= b;
	}

	if (neg) {
		if( width && (pad & PAD_ZERO) ) {
			printchar (uart, '-');
			++pc;
			--width;
		}
		else {
			*--s = '-';
		}
	}

	return pc + prints (uart, s, width, pad);
}

static int print(struct _uart *uart, const char *format, va_list args )
{
	register int width, pad;
	register int pc = 0;
	char scr[2];

	for (; *format != 0; ++format) {
		if (*format == '%') {
			++format;
			width = pad = 0;
			if (*format == '\0') break;
			if (*format == '%') goto out;
			if (*format == '-') {
				++format;
				pad = PAD_RIGHT;
			}
			while (*format == '0') {
				++format;
				pad |= PAD_ZERO;
			}
			for ( ; *format >= '0' && *format <= '9'; ++format) {
				width *= 10;
				width += *format - '0';
			}
			if( *format == 's' ) {
				register char *s = (char *)va_arg( args, int );
				pc += prints (uart, s?s:"(null)", width, pad);
				continue;
			}
			if( *format == 'd' ) {
				pc += printi (uart, va_arg( args, int ), 10, 1, width, pad, 'a');
				continue;
			}
			if( *format == 'x' ) {
				pc += printi (uart, va_arg( args, int ), 16, 0, width, pad, 'a');
				continue;
			}
			if( *format == 'X' ) {
				pc += printi (uart, va_arg( args, int ), 16, 0, width, pad, 'A');
				continue;
			}
			if( *format == 'u' ) {
				pc += printi (uart, va_arg( args, int ), 10, 0, width, pad, 'a');
				continue;
			}
			if( *format == 'c' ) {
				/* char are converted to int then pushed on the stack */
				scr[0] = (char)va_arg( args, int );
				scr[1] = '\0';
				pc += prints (uart, scr, width, pad);
				continue;
			}
		}
		else {
		out:
			printchar (uart, *format);
			++pc;
		}
	}
	return pc;
}


