/*
 * wiflyFreeRTOS.c
 *
 *  Created on: 03/12/2012
 *      Author: Javier Rufas
 *
 *
 *      WiFly abstraction layer of wifly library for FreeRTOS
 */


#include "wiflyFreeRTOS.h"
#include "task.h"
#include "strutils.h"
#include <string.h>

#define TMP_BUFFER_SIZE 100

static char *sOk = "AOK";
//static char *sOpen = "*OPEN*";
/*
 * Time counter for WiFly driver
 *
 */
unsigned long millis()
{
	return (unsigned long) xTaskGetTickCount()/portTICK_RATE_MS;
}

/*
 * Debug function for WiFly driver
 *
 */
void DebugWifly(void *debugParam, char *text, int len, int mode)
{
	struct _wiflyFreeRTOS *wiflyFreeRTOS = (struct _wiflyFreeRTOS *) debugParam;
	if( mode <= wiflyFreeRTOS->wifly.bDebugMode) {
		/* Take UART Mutex */
		if( xSemaphoreTake( wiflyFreeRTOS->xMutexUartDebug, portMAX_DELAY ) == pdTRUE )
		{
			// UARTsendString(&(wiflyFreeRTOS->uartDebug), (byte *) "DB->");
			while(len) {
				UARTsendBytes(&(wiflyFreeRTOS->uartDebug), (byte *) text, (uint32_t) 1);
				text++;
				len--;
			}
			// UARTsendString(&(wiflyFreeRTOS->uartDebug), (byte *) "<-DB\n");
			/* Release UART Mutex */
			xSemaphoreGive( wiflyFreeRTOS->xMutexUartDebug );
		}
	}
}

/*
struct _wiflyFreeRTOS {
	struct _wifly wifly;
	xSemaphoreHandle xMutexWifly;
	struct _uart uartDebug;
	xSemaphoreHandle xMutexUartDebug;
};
 */

/*
 * Pre-initialization utility routines
 * Setting UART parameters
 *
 */
void WiFlyFreeRTOSsetUartDebug(struct _wiflyFreeRTOS *wiflyFreeRTOS, int port, int baudrate, int mode)
{
	wiflyFreeRTOS->uartDebug.port = port;
	wiflyFreeRTOS->uartDebug.baudrate = baudrate;
	wiflyFreeRTOS->wifly.bDebugMode = mode;
}

void WiFlyFreeRTOSsetWiFlyUart(struct _wiflyFreeRTOS *wiflyFreeRTOS, int port)
{
	wiflyFreeRTOS->wifly.uart.port = port;
}

/*
 * Reset WiFly values to default for application
 *
 */
void resetValues(struct _wiflyFreeRTOS *wiflyFreeRTOS) {
	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);

	WiFlysendCommandSimple(pwifly, "set dns name 0", sOk);
	WiFlysendCommandSimple(pwifly, "set ip host 0", sOk);

	WiFlysendCommandSimple(pwifly, "set com remote *HELLO*", sOk);
	WiFlysendCommandSimple(pwifly, "set ip proto 2", sOk);
	WiFlysendCommandSimple(pwifly, "set option format 0", sOk);
}

/*
 * Initializing WiFlyFreeRTOS structure data
 * - Init wifly
 * - Init uartDebug
 * - Create mutexes
 *
 * returns true on success, false otherwise
 */
boolean WiFlyFreeRTOSbegin(struct _wiflyFreeRTOS *wiflyFreeRTOS) {

	boolean returnValue = false;
	/* Create semaphores and queues */
	wiflyFreeRTOS->xMutexUartDebug = xSemaphoreCreateMutex();
	wiflyFreeRTOS->xMutexWifly = xSemaphoreCreateMutex();
	wiflyFreeRTOS->xQueueTerminalRead = xQueueCreate( terminalQUEUE_LENGTH, terminalQUEUE_DATA_TYPE );
	wiflyFreeRTOS->xQueueTerminalWrite = xQueueCreate( terminalQUEUE_LENGTH, terminalQUEUE_DATA_TYPE );
	wiflyFreeRTOS->xQueueDataRead = xQueueCreate( dataQUEUE_LENGTH, dataQUEUE_DATA_TYPE );
	wiflyFreeRTOS->xQueueDataWrite = xQueueCreate( dataQUEUE_LENGTH, dataQUEUE_DATA_TYPE );

	/* On success */
	if( (wiflyFreeRTOS->xMutexUartDebug != NULL) &&
			(wiflyFreeRTOS->xMutexWifly != NULL) &&
			(wiflyFreeRTOS->xQueueTerminalRead != NULL) &&
			(wiflyFreeRTOS->xQueueTerminalWrite != NULL) &&
			(wiflyFreeRTOS->xQueueDataRead != NULL) &&
			(wiflyFreeRTOS->xQueueDataWrite != NULL) ) {

		/* Init UART Debug */
		if( xSemaphoreTake( wiflyFreeRTOS->xMutexUartDebug, portMAX_DELAY ) == pdTRUE ) {
			UARTinit(&(wiflyFreeRTOS->uartDebug));
			xSemaphoreGive( wiflyFreeRTOS->xMutexUartDebug );
		}

		/* Init WiFly */
		if( xSemaphoreTake( wiflyFreeRTOS->xMutexWifly, portMAX_DELAY ) == pdTRUE ) {
			wiflyFreeRTOS->wifly.millis = millis;
			wiflyFreeRTOS->wifly.debugParam = (void *) wiflyFreeRTOS;
			wiflyFreeRTOS->wifly.Debug = DebugWifly;
			returnValue = WiFlybegin(&(wiflyFreeRTOS->wifly));
			resetValues(wiflyFreeRTOS);
			xSemaphoreGive( wiflyFreeRTOS->xMutexWifly );
		}
	}
	return returnValue;
}

/*
 * Default joining to a WPA network
 * Parameters: ssid and passphrase
 */

boolean WiFlyFreeRTOSjoinAutoWPA(struct _wiflyFreeRTOS *wiflyFreeRTOS, char *ssid, char *passphrase)
{
	int iTry = 0;
	boolean bJoined = false;

	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);
	if( xSemaphoreTake( wiflyFreeRTOS->xMutexWifly, portMAX_DELAY ) == pdTRUE ) {
		WiFlysetJoinMode( pwifly, WIFLY_JOIN_AUTO );
		WiFlysetDHCPMode( pwifly, WIFLY_DHCP_ON );
		WiFlysetAuthMode( pwifly, WIFLY_AUTH_WPA1);
		WiFlysetPassphrase(pwifly, passphrase);
		WiFlyleave(pwifly);
		while(iTry++ < 3) {
			if((bJoined=WiFlyjoinSSID(pwifly, ssid)))
				break;
		}

		xSemaphoreGive( wiflyFreeRTOS->xMutexWifly );
	}

	return bJoined;
}

/*
 * Returns the association state
 */

boolean WiFlyFreeRTOSisAssociated(struct _wiflyFreeRTOS *wiflyFreeRTOS)
{
	boolean returnValue = false;
	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);
	if( xSemaphoreTake( wiflyFreeRTOS->xMutexWifly, portMAX_DELAY ) == pdTRUE ) {
		WiFlygetDeviceStatus(pwifly);
		returnValue=WiFlyisAssociated(pwifly);
		xSemaphoreGive( wiflyFreeRTOS->xMutexWifly );
	}
	return returnValue;
}

void WiFlyFreeRTOSemptyQueues(struct _wiflyFreeRTOS *wiflyFreeRTOS)
{
	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);
	int item;
	while( uxQueueMessagesWaiting(wiflyFreeRTOS->xQueueDataRead) ){
		xQueueReceive(wiflyFreeRTOS->xQueueDataRead, &item, 0);
	}
	while( uxQueueMessagesWaiting(wiflyFreeRTOS->xQueueDataWrite) ){
		xQueueReceive(wiflyFreeRTOS->xQueueDataWrite, &item, 0);
	}
	while( uxQueueMessagesWaiting(wiflyFreeRTOS->xQueueTerminalRead) ){
		xQueueReceive(wiflyFreeRTOS->xQueueTerminalRead, &item, 0);
	}
	while( uxQueueMessagesWaiting(wiflyFreeRTOS->xQueueTerminalWrite) ){
		xQueueReceive(wiflyFreeRTOS->xQueueTerminalWrite, &item, 0);
	}
}

/***************************************************************************
 * SKELETON FUNCTION. Take MutEx / Process / Release MutEx
 ***************************************************************************
boolean WiFlyFreeRTOSskeletonFunction(struct _wiflyFreeRTOS *wiflyFreeRTOS)
{
	boolean returnValue = false;
	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);
	struct _uart *puart = &(pwifly->uart);

	if( xSemaphoreTake( wiflyFreeRTOS->xMutexWifly, portMAX_DELAY ) == pdTRUE ) {

		xSemaphoreGive( wiflyFreeRTOS->xMutexWifly );
		returnValue = true;
	}

	return returnValue;
}
 ***************************************************************************
 ***************************************************************************
 ***************************************************************************/

/*
 * Listens from WiFly terminal for an *OPEN* command.
 * then send received data to xQueueTerminalRead
 * and listen from xQueueTerminalWrite to sending data to WiFly terminal
 */
boolean WiFlyFreeRTOSterminalListen(struct _wiflyFreeRTOS *wiflyFreeRTOS)
{
	boolean returnValue = false;
	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);
	struct _uart *puart = &(pwifly->uart);

	unsigned long bufferSize = TMP_BUFFER_SIZE;
	char tmpBuffer[bufferSize];
	char *current = tmpBuffer;
	int nchars;
	char ip[17];

	if( xSemaphoreTake( wiflyFreeRTOS->xMutexWifly, portMAX_DELAY ) == pdTRUE ) {

		UARTflush(puart);

		WiFlygetIP(pwifly, tmpBuffer, TMP_BUFFER_SIZE);
		if( (current=strchr(tmpBuffer, ':')) != NULL ) {
			*current='\0';
		}
		strncpy(ip, tmpBuffer, 16);

		current = tmpBuffer;
		strcpy(tmpBuffer, "Waiting for a connection on IP ");
		strcat(tmpBuffer, ip);
		strcat(tmpBuffer, ":2000");
		DebugWifly(pwifly->debugParam, tmpBuffer, strlen(tmpBuffer), DEBUG_MODE_NONE);

		while(!WiFlyserveConnection(pwifly,SERVING_WAIT_TIME) ); // Wait for a connection.

		WiFlygetDeviceStatus(pwifly);
		while(WiFlyisTCPConnected(pwifly)) {
			WiFlyexitCommandMode(pwifly);
			UARTflush(puart);
			UARTsendString(puart, (byte *) "\nWaiting for a command (get, set, run)\n");
			UARTflush(puart);

			while(!UARTisAvailable(puart)); //wait for a command line

			WiFlyDelayUntilReceivingData(pwifly, 50);

			current = tmpBuffer;
			while(UARTisAvailable(puart)) {
				UARTreadByte(puart, (byte *) current);
				if(*current == '\n' || *current == '\r') {
					*current = '\0';
					break;
				} else {
					current ++;
				}
			}
			/* If command line isn't empty (avoid extra CR or LF) */
			if(current != tmpBuffer) {
				/* On *CLOS* detected, exit */
				if( !strcmp( tmpBuffer, WiFlygetFixedPrompt(WIFLY_MSG_CLOSE) )) {
					break;
				}
				nchars = 0;
				current = tmpBuffer;
				while( true ) {
					if( xQueueSend( wiflyFreeRTOS->xQueueTerminalRead, current, portMAX_DELAY ) != pdTRUE ) {
						break; /* Cancel on error */
					}
					if( *current == '\0')
						break;

					current++;
				}
				current = tmpBuffer;
				while( true ) {
					if( xQueueReceive( wiflyFreeRTOS->xQueueTerminalWrite, current, portMAX_DELAY ) != pdTRUE ) {
						break; /* Cancel on error */
					}
					if( *current != '\0') {
						UARTsendByte(puart, (byte) *current);
					} else {
						UARTsendByte(puart, (byte) '\n');
						break;
					}
					if( nchars < TMP_BUFFER_SIZE ) current++;
					nchars++;
					UARTflush(puart);
				}
				*current = '\0';
				if(! strcmp(tmpBuffer, "GO")) { // on RUN command, exit.
					returnValue = true;
					break;
				}
			}
			WiFlygetDeviceStatus(pwifly);
		}
		xSemaphoreGive( wiflyFreeRTOS->xMutexWifly );
	}
	return returnValue;
}

/*
 * Sends HTTP Request to arpalab.appspot.com
 * using semaphore to ensure mutual exclusive access to WiFly
 *
 * PARAMETERS:
 * wiflyFreeRTOS wiflyFreeRTOS structure pointer
 * url			 url to GET
 * receiveBuffer buffer of byte to receive data
 * maxLenght     buffer length
 */

unsigned long WiFlyFreeRTOSgetHTTPRequestOLD(struct _wiflyFreeRTOS *wiflyFreeRTOS, char* url, byte *receiveBuffer, unsigned long maxLength)
{
	boolean success = false;
	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);
	struct _uart *puart = &(pwifly->uart);
	unsigned long bufferSize = TMP_BUFFER_SIZE;

	char tmpBuffer[bufferSize];
	int iTry = 0;

	unsigned long bytesReaded = 0L;
	//	unsigned char *pData = (unsigned char *) tmpBuffer;
	//	int data;
	char *root;


	if( !strncmpi(url, "http://", 7) ) url+=7; // remove "http://" if exist

	root = strstr(url, "/");
	if( root == NULL ) root="/";

	/* Take WiFly Mutex */
	if( xSemaphoreTake( wiflyFreeRTOS->xMutexWifly, portMAX_DELAY ) == pdTRUE ) {
		/* The following lines will only execute once the semaphore has been
		successfully obtained - so WiFly uart can be accessed freely. */
		tmpBuffer[0]='\0';
		WiFlygetDeviceStatus(pwifly);
		/*
		if( ! WiFlyisAssociated(pwifly)) {
			WiFlyleave(pwifly);
			WiFlyjoin(pwifly);
		}
		 */
		if( WiFlyisAssociated(pwifly)) {
			if(WiFlyisTCPConnected(pwifly)) {
				WiFlycloseConnection(pwifly, true);
			}
			strcpy(tmpBuffer, "set dns na ");
			//strcpy(tmpBuffer, "open ");
			*root='\0';

			strcat(tmpBuffer, url); // host
			*root='/';

			//WiFlysendCommandSimple(pwifly, "set ip remote 80", sOk);
			//WiFlysendCommandSimple(pwifly, "set ip proto 18", sOk);
			//WiFlysendCommandSimple(pwifly, "set com remote 0", sOk);
			WiFlysendCommandSimple(pwifly, tmpBuffer, sOk);

			UARTflush(&(pwifly->uart));

			while( !success && iTry < 5) { // Send GET Try it 5 times

				if(WiFlysendHTTPGet(pwifly, root, (char*) receiveBuffer, maxLength, 5000)) {

					//... and receive results.
					while(!UARTisAvailable(puart));
					// Analise headers

					while(UARTreadLine(puart, (byte *) tmpBuffer, (uint32_t) bufferSize)){
						DebugWifly(NULL, tmpBuffer, strlen(tmpBuffer), DEBUG_MODE_FULL);
						if( strstr(tmpBuffer, "200 OK") ) success = true;
						if( strlen(tmpBuffer) == 0 ) break; // end of headers = beginning of data
					}
					if(success) { // On success, send data to Data read queue
						if(receiveBuffer != NULL) {
							while(UARTisAvailable(puart) && bytesReaded < maxLength) {
								if( UARTreadByte(puart, (byte *) receiveBuffer++) ) {
									bytesReaded++;
								}
							}
						} else {
							bytesReaded=1; // If no buffer, return success
						}
					} else {
						UARTflush(puart);
					}
				}
				iTry++;
			}
		}
		resetValues(wiflyFreeRTOS);

		/* Release WiFly Mutex */
		xSemaphoreGive( wiflyFreeRTOS->xMutexWifly );
	}
	return bytesReaded;
}

/*
 *
 * Sends HTTP Request to arpalab.appspot.com
 * using semaphore to ensure mutual exclusive access to WiFly
 *
 * PARAMETERS:
 * wiflyFreeRTOS wiflyFreeRTOS structure pointer
 * url			 url to GET
 * receiveBuffer buffer of byte to receive data
 * maxLenght     buffer length
 */

unsigned long WiFlyFreeRTOSgetHTTPRequest(struct _wiflyFreeRTOS *wiflyFreeRTOS, char* url, byte *receiveBuffer, unsigned long maxLength)
{
	struct _wifly *pwifly = &(wiflyFreeRTOS->wifly);
	int iTry = 0;
	unsigned long bytesReaded = 0L;


	/* Take WiFly Mutex */
	if( xSemaphoreTake( wiflyFreeRTOS->xMutexWifly, portMAX_DELAY ) == pdTRUE ) {
		/* The following lines will only execute once the semaphore has been
		successfully obtained - so WiFly uart can be accessed freely. */
		WiFlygetDeviceStatus(pwifly);
		if( WiFlyisAssociated(pwifly)) {
			if(WiFlyisTCPConnected(pwifly)) {
				WiFlycloseConnection(pwifly, true);
			}

			UARTflush(&(pwifly->uart));

			while( bytesReaded < 1 && iTry < 5) { // Send GET Try it 5 times

				bytesReaded = WiFlygetDataHTTPGet(pwifly, url, (char*) receiveBuffer, maxLength, 15000);
				iTry++;
			}
		}
		resetValues(wiflyFreeRTOS);

		/* Release WiFly Mutex */
		xSemaphoreGive( wiflyFreeRTOS->xMutexWifly );
	}
	return bytesReaded;
}

